import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import TestUUID as TestUUID

# Module: TestUTF8

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def TestEncodeHappyCase():
        d_0_unicodeString_: _dafny.Seq
        d_0_unicodeString_ = _dafny.Seq("abc\u0306\u01FD\u03B2")
        d_1_expectedBytes_: _dafny.Seq
        d_1_expectedBytes_ = _dafny.Seq([97, 98, 99, 204, 134, 199, 189, 206, 178])
        d_2_valueOrError0_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_2_valueOrError0_ = UTF8.default__.Encode(d_0_unicodeString_)
        if not(not((d_2_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(14,19): " + _dafny.string_of(d_2_valueOrError0_))
        d_3_encoded_: _dafny.Seq
        d_3_encoded_ = (d_2_valueOrError0_).Extract()
        if not((d_1_expectedBytes_) == (d_3_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(15,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def TestEncodeInvalidUnicode():
        d_0_invalidUnicode_: _dafny.Seq
        d_0_invalidUnicode_ = _dafny.Seq("abc\uD800")
        d_1_encoded_: Wrappers.Result
        d_1_encoded_ = UTF8.default__.Encode(d_0_invalidUnicode_)
        if not((d_1_encoded_).is_Failure):
            raise _dafny.HaltException("test/UTF8.dfy(22,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def TestDecodeHappyCase():
        d_0_unicodeBytes_: _dafny.Seq
        d_0_unicodeBytes_ = _dafny.Seq([97, 98, 99, 204, 134, 199, 189, 206, 178])
        d_1_expectedString_: _dafny.Seq
        d_1_expectedString_ = _dafny.Seq("abc\u0306\u01FD\u03B2")
        d_2_valueOrError0_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_2_valueOrError0_ = UTF8.default__.Decode(d_0_unicodeBytes_)
        if not(not((d_2_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(32,19): " + _dafny.string_of(d_2_valueOrError0_))
        d_3_decoded_: _dafny.Seq
        d_3_decoded_ = (d_2_valueOrError0_).Extract()
        if not((d_1_expectedString_) == (d_3_decoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(33,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def TestDecodeInvalidUnicode():
        d_0_invalidUnicode_: _dafny.Seq
        d_0_invalidUnicode_ = _dafny.Seq([97, 98, 99, 237, 160, 128])
        if not(not(UTF8.default__.ValidUTF8Seq(d_0_invalidUnicode_))):
            raise _dafny.HaltException("test/UTF8.dfy(39,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((UTF8.default__.Decode(d_0_invalidUnicode_)).is_Failure):
            raise _dafny.HaltException("test/UTF8.dfy(40,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def Test1Byte():
        d_0_decoded_: _dafny.Seq
        d_0_decoded_ = _dafny.Seq("\u0000")
        d_1_valueOrError0_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_1_valueOrError0_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_1_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(46,19): " + _dafny.string_of(d_1_valueOrError0_))
        d_2_encoded_: _dafny.Seq
        d_2_encoded_ = (d_1_valueOrError0_).Extract()
        if not((_dafny.Seq([0])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(47,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses1Byte(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(48,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_3_valueOrError1_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_3_valueOrError1_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_3_valueOrError1_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(49,21): " + _dafny.string_of(d_3_valueOrError1_))
        d_4_redecoded_: _dafny.Seq
        d_4_redecoded_ = (d_3_valueOrError1_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(50,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\u0020")
        d_5_valueOrError2_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_5_valueOrError2_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_5_valueOrError2_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(54,15): " + _dafny.string_of(d_5_valueOrError2_))
        d_2_encoded_ = (d_5_valueOrError2_).Extract()
        if not((_dafny.Seq([32])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(55,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses1Byte(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(56,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_6_valueOrError3_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_6_valueOrError3_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_6_valueOrError3_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(57,17): " + _dafny.string_of(d_6_valueOrError3_))
        d_4_redecoded_ = (d_6_valueOrError3_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(58,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("$")
        d_7_valueOrError4_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_7_valueOrError4_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_7_valueOrError4_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(61,15): " + _dafny.string_of(d_7_valueOrError4_))
        d_2_encoded_ = (d_7_valueOrError4_).Extract()
        if not((_dafny.Seq([36])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(62,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses1Byte(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(63,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_8_valueOrError5_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_8_valueOrError5_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_8_valueOrError5_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(64,17): " + _dafny.string_of(d_8_valueOrError5_))
        d_4_redecoded_ = (d_8_valueOrError5_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(65,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("0")
        d_9_valueOrError6_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_9_valueOrError6_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_9_valueOrError6_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(68,15): " + _dafny.string_of(d_9_valueOrError6_))
        d_2_encoded_ = (d_9_valueOrError6_).Extract()
        if not((_dafny.Seq([48])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(69,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses1Byte(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(70,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_10_valueOrError7_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_10_valueOrError7_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_10_valueOrError7_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(71,17): " + _dafny.string_of(d_10_valueOrError7_))
        d_4_redecoded_ = (d_10_valueOrError7_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(72,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("A")
        d_11_valueOrError8_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_11_valueOrError8_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_11_valueOrError8_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(75,15): " + _dafny.string_of(d_11_valueOrError8_))
        d_2_encoded_ = (d_11_valueOrError8_).Extract()
        if not((_dafny.Seq([65])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(76,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses1Byte(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(77,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_12_valueOrError9_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_12_valueOrError9_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_12_valueOrError9_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(78,17): " + _dafny.string_of(d_12_valueOrError9_))
        d_4_redecoded_ = (d_12_valueOrError9_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(79,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("a")
        d_13_valueOrError10_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_13_valueOrError10_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_13_valueOrError10_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(82,15): " + _dafny.string_of(d_13_valueOrError10_))
        d_2_encoded_ = (d_13_valueOrError10_).Extract()
        if not((_dafny.Seq([97])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(83,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses1Byte(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(84,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_14_valueOrError11_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_14_valueOrError11_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_14_valueOrError11_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(85,17): " + _dafny.string_of(d_14_valueOrError11_))
        d_4_redecoded_ = (d_14_valueOrError11_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(86,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def Test2Bytes():
        d_0_decoded_: _dafny.Seq
        d_0_decoded_ = _dafny.Seq("\u00A3")
        d_1_valueOrError0_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_1_valueOrError0_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_1_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(92,19): " + _dafny.string_of(d_1_valueOrError0_))
        d_2_encoded_: _dafny.Seq
        d_2_encoded_ = (d_1_valueOrError0_).Extract()
        if not((_dafny.Seq([194, 163])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(93,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses2Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(94,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_3_valueOrError1_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_3_valueOrError1_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_3_valueOrError1_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(95,21): " + _dafny.string_of(d_3_valueOrError1_))
        d_4_redecoded_: _dafny.Seq
        d_4_redecoded_ = (d_3_valueOrError1_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(96,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\u00A9")
        d_5_valueOrError2_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_5_valueOrError2_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_5_valueOrError2_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(100,15): " + _dafny.string_of(d_5_valueOrError2_))
        d_2_encoded_ = (d_5_valueOrError2_).Extract()
        if not((_dafny.Seq([194, 169])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(101,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses2Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(102,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_6_valueOrError3_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_6_valueOrError3_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_6_valueOrError3_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(103,17): " + _dafny.string_of(d_6_valueOrError3_))
        d_4_redecoded_ = (d_6_valueOrError3_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(104,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\u00AE")
        d_7_valueOrError4_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_7_valueOrError4_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_7_valueOrError4_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(108,15): " + _dafny.string_of(d_7_valueOrError4_))
        d_2_encoded_ = (d_7_valueOrError4_).Extract()
        if not((_dafny.Seq([194, 174])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(109,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses2Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(110,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_8_valueOrError5_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_8_valueOrError5_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_8_valueOrError5_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(111,17): " + _dafny.string_of(d_8_valueOrError5_))
        d_4_redecoded_ = (d_8_valueOrError5_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(112,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\u03C0")
        d_9_valueOrError6_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_9_valueOrError6_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_9_valueOrError6_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(116,15): " + _dafny.string_of(d_9_valueOrError6_))
        d_2_encoded_ = (d_9_valueOrError6_).Extract()
        if not((_dafny.Seq([207, 128])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(117,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses2Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(118,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_10_valueOrError7_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_10_valueOrError7_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_10_valueOrError7_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(119,17): " + _dafny.string_of(d_10_valueOrError7_))
        d_4_redecoded_ = (d_10_valueOrError7_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(120,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def Test3Bytes():
        d_0_decoded_: _dafny.Seq
        d_0_decoded_ = _dafny.Seq("\u2386")
        d_1_valueOrError0_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_1_valueOrError0_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_1_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(126,19): " + _dafny.string_of(d_1_valueOrError0_))
        d_2_encoded_: _dafny.Seq
        d_2_encoded_ = (d_1_valueOrError0_).Extract()
        if not((_dafny.Seq([226, 142, 134])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(127,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses3Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(128,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_3_valueOrError1_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_3_valueOrError1_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_3_valueOrError1_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(129,21): " + _dafny.string_of(d_3_valueOrError1_))
        d_4_redecoded_: _dafny.Seq
        d_4_redecoded_ = (d_3_valueOrError1_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(130,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\u2387")
        d_5_valueOrError2_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_5_valueOrError2_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_5_valueOrError2_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(134,15): " + _dafny.string_of(d_5_valueOrError2_))
        d_2_encoded_ = (d_5_valueOrError2_).Extract()
        if not((_dafny.Seq([226, 142, 135])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(135,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses3Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(136,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_6_valueOrError3_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_6_valueOrError3_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_6_valueOrError3_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(137,17): " + _dafny.string_of(d_6_valueOrError3_))
        d_4_redecoded_ = (d_6_valueOrError3_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(138,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\u231B")
        d_7_valueOrError4_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_7_valueOrError4_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_7_valueOrError4_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(142,15): " + _dafny.string_of(d_7_valueOrError4_))
        d_2_encoded_ = (d_7_valueOrError4_).Extract()
        if not((_dafny.Seq([226, 140, 155])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(143,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses3Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(144,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_8_valueOrError5_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_8_valueOrError5_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_8_valueOrError5_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(145,17): " + _dafny.string_of(d_8_valueOrError5_))
        d_4_redecoded_ = (d_8_valueOrError5_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(146,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\u1D78")
        d_9_valueOrError6_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_9_valueOrError6_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_9_valueOrError6_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(150,15): " + _dafny.string_of(d_9_valueOrError6_))
        d_2_encoded_ = (d_9_valueOrError6_).Extract()
        if not((_dafny.Seq([225, 181, 184])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(151,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses3Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(152,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_10_valueOrError7_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_10_valueOrError7_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_10_valueOrError7_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(153,17): " + _dafny.string_of(d_10_valueOrError7_))
        d_4_redecoded_ = (d_10_valueOrError7_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(154,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\u732B")
        d_11_valueOrError8_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_11_valueOrError8_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_11_valueOrError8_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(158,15): " + _dafny.string_of(d_11_valueOrError8_))
        d_2_encoded_ = (d_11_valueOrError8_).Extract()
        if not((_dafny.Seq([231, 140, 171])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(159,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses3Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(160,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_12_valueOrError9_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_12_valueOrError9_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_12_valueOrError9_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(161,17): " + _dafny.string_of(d_12_valueOrError9_))
        d_4_redecoded_ = (d_12_valueOrError9_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(162,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def Test4Bytes():
        d_0_decoded_: _dafny.Seq
        d_0_decoded_ = _dafny.Seq("\uD808\uDC00")
        d_1_valueOrError0_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_1_valueOrError0_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_1_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(168,19): " + _dafny.string_of(d_1_valueOrError0_))
        d_2_encoded_: _dafny.Seq
        d_2_encoded_ = (d_1_valueOrError0_).Extract()
        if not((_dafny.Seq([240, 146, 128, 128])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(169,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses4Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(170,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_3_valueOrError1_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_3_valueOrError1_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_3_valueOrError1_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(171,21): " + _dafny.string_of(d_3_valueOrError1_))
        d_4_redecoded_: _dafny.Seq
        d_4_redecoded_ = (d_3_valueOrError1_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(172,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_0_decoded_ = _dafny.Seq("\uD835\uDFC1")
        d_5_valueOrError2_: Wrappers.Result = Wrappers.Result.default(UTF8.ValidUTF8Bytes.default)()
        d_5_valueOrError2_ = UTF8.default__.Encode(d_0_decoded_)
        if not(not((d_5_valueOrError2_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(176,15): " + _dafny.string_of(d_5_valueOrError2_))
        d_2_encoded_ = (d_5_valueOrError2_).Extract()
        if not((_dafny.Seq([240, 157, 159, 129])) == (d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(177,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(UTF8.default__.Uses4Bytes(d_2_encoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(178,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_6_valueOrError3_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_6_valueOrError3_ = UTF8.default__.Decode(d_2_encoded_)
        if not(not((d_6_valueOrError3_).IsFailure())):
            raise _dafny.HaltException("test/UTF8.dfy(179,17): " + _dafny.string_of(d_6_valueOrError3_))
        d_4_redecoded_ = (d_6_valueOrError3_).Extract()
        if not((d_0_decoded_) == (d_4_redecoded_)):
            raise _dafny.HaltException("test/UTF8.dfy(180,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import aws_cryptography_internal_kms.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import aws_cryptography_internal_kms.internaldafny.generated.ComAmazonawsKmsTypes as ComAmazonawsKmsTypes
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Vectors as JSON_Utils_Vectors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Errors as JSON_Errors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Values as JSON_Values
import smithy_dafny_standard_library.internaldafny.generated.JSON_Spec as JSON_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_Grammar as JSON_Grammar
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer_ByteStrConversion as JSON_Serializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Serializer as JSON_Serializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_Uint16StrConversion as JSON_Deserializer_Uint16StrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer_ByteStrConversion as JSON_Deserializer_ByteStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Deserializer as JSON_Deserializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax_Spec as JSON_ConcreteSyntax_Spec
import smithy_dafny_standard_library.internaldafny.generated.JSON_ConcreteSyntax_SpecProperties as JSON_ConcreteSyntax_SpecProperties
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Serializer as JSON_ZeroCopy_Serializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Core as JSON_ZeroCopy_Deserializer_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Strings as JSON_ZeroCopy_Deserializer_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Numbers as JSON_ZeroCopy_Deserializer_Numbers
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_ObjectParams as JSON_ZeroCopy_Deserializer_ObjectParams
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Objects as JSON_ZeroCopy_Deserializer_Objects
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_ArrayParams as JSON_ZeroCopy_Deserializer_ArrayParams
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Arrays as JSON_ZeroCopy_Deserializer_Arrays
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Constants as JSON_ZeroCopy_Deserializer_Constants
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_Values as JSON_ZeroCopy_Deserializer_Values
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer_API as JSON_ZeroCopy_Deserializer_API
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_Deserializer as JSON_ZeroCopy_Deserializer
import smithy_dafny_standard_library.internaldafny.generated.JSON_ZeroCopy_API as JSON_ZeroCopy_API
import smithy_dafny_standard_library.internaldafny.generated.JSON_API as JSON_API
import aws_cryptography_internal_kms.internaldafny.generated.Com_Amazonaws_Kms as Com_Amazonaws_Kms
import aws_cryptography_internal_kms.internaldafny.generated.Com_Amazonaws as Com_Amazonaws

# Module: Com




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import aws_cryptography_internal_kms.internaldafny.generated.ComAmazonawsKmsTypes as ComAmazonawsKmsTypes
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import aws_cryptography_internal_kms.internaldafny.generated.Com_Amazonaws_Kms as Com_Amazonaws_Kms

# Module: TestComAmazonawsKms

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def workAround(literal):
        return literal

    @staticmethod
    def BasicDecryptTests():
        d_0_CiphertextBlob_: _dafny.Seq
        d_0_CiphertextBlob_ = _dafny.Seq([1, 1, 1, 0, 120, 64, 243, 140, 39, 94, 49, 9, 116, 22, 193, 7, 41, 81, 80, 87, 25, 100, 173, 163, 239, 28, 33, 233, 76, 139, 160, 189, 188, 157, 15, 180, 20, 0, 0, 0, 98, 48, 96, 6, 9, 42, 134, 72, 134, 247, 13, 1, 7, 6, 160, 83, 48, 81, 2, 1, 0, 48, 76, 6, 9, 42, 134, 72, 134, 247, 13, 1, 7, 1, 48, 30, 6, 9, 96, 134, 72, 1, 101, 3, 4, 1, 46, 48, 17, 4, 12, 196, 249, 60, 7, 21, 231, 87, 70, 216, 12, 31, 13, 2, 1, 16, 128, 31, 222, 119, 162, 112, 88, 153, 39, 197, 21, 182, 116, 176, 120, 174, 107, 82, 182, 223, 160, 201, 15, 29, 3, 254, 3, 208, 72, 171, 64, 207, 175])
        default__.BasicDecryptTest(ComAmazonawsKmsTypes.DecryptRequest_DecryptRequest(default__.workAround(d_0_CiphertextBlob_), Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_Some(default__.keyId), Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None()), _dafny.Seq([165, 191, 67, 62]), default__.keyId)

    @staticmethod
    def BasicGenerateTests():
        default__.BasicGenerateTest(ComAmazonawsKmsTypes.GenerateDataKeyRequest_GenerateDataKeyRequest(default__.keyId, Wrappers.Option_None(), Wrappers.Option_Some(32), Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None()))

    @staticmethod
    def BasicGenerateWithoutPlaintextTests():
        default__.BasicGenerateWithoutPlaintextTest(ComAmazonawsKmsTypes.GenerateDataKeyWithoutPlaintextRequest_GenerateDataKeyWithoutPlaintextRequest(default__.keyIdGenerateWOPlain, Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_Some(32), Wrappers.Option_None(), Wrappers.Option_None()))

    @staticmethod
    def BasicEncryptTests():
        default__.BasicEncryptTest(ComAmazonawsKmsTypes.EncryptRequest_EncryptRequest(default__.keyId, _dafny.Seq([97, 115, 100, 102]), Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None()))

    @staticmethod
    def BasicFailTests():
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(default__.TEST__REGION)
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(117,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()
        d_2_ret_: Wrappers.Result
        out1_: Wrappers.Result
        out1_ = (d_1_client_).GenerateDataKeyWithoutPlaintext(default__.failingInput)
        d_2_ret_ = out1_
        if not((d_2_ret_).is_Failure):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(119,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_3_err_: ComAmazonawsKmsTypes.Error
        d_3_err_ = (d_2_ret_).error
        if not((d_3_err_).is_Opaque):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(121,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        source0_ = d_3_err_
        with _dafny.label("match0"):
            if True:
                if source0_.is_Opaque:
                    d_4_obj_ = source0_.obj
                    d_5_alt__text_ = source0_.alt__text
                    if not(True):
                        raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(123,36): " + _dafny.string_of(_dafny.Seq("expectation violation")))
                    raise _dafny.Break("match0")
            if True:
                if not(False):
                    raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(124,16): " + _dafny.string_of(_dafny.Seq("Failing KMS Key MUST cause an OpaqueError that can later be unwrapped to a proper but generic KMS Exception.")))
            pass

    @staticmethod
    def BasicDecryptTest(input, expectedPlaintext, expectedKeyId):
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(default__.TEST__REGION)
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(134,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()
        d_2_ret_: Wrappers.Result
        out1_: Wrappers.Result
        out1_ = (d_1_client_).Decrypt(input)
        d_2_ret_ = out1_
        if not((d_2_ret_).is_Success):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(140,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        let_tmp_rhs0_ = (d_2_ret_).value
        d_3_KeyId_ = let_tmp_rhs0_.KeyId
        d_4_Plaintext_ = let_tmp_rhs0_.Plaintext
        d_5_EncryptionAlgorithm_ = let_tmp_rhs0_.EncryptionAlgorithm
        d_6_CiphertextBlob_ = let_tmp_rhs0_.CiphertextForRecipient
        if not((d_4_Plaintext_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(144,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((d_3_KeyId_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(145,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(((d_4_Plaintext_).value) == (expectedPlaintext)):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(146,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(((d_3_KeyId_).value) == (expectedKeyId)):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(147,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def BasicGenerateTest(input):
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(default__.TEST__REGION)
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(155,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()
        d_2_ret_: Wrappers.Result
        out1_: Wrappers.Result
        out1_ = (d_1_client_).GenerateDataKey(input)
        d_2_ret_ = out1_
        if not((d_2_ret_).is_Success):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(159,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        let_tmp_rhs0_ = (d_2_ret_).value
        d_3_CiphertextBlob_ = let_tmp_rhs0_.CiphertextBlob
        d_4_Plaintext_ = let_tmp_rhs0_.Plaintext
        d_5_KeyId_ = let_tmp_rhs0_.KeyId
        d_6_CiphertextForRecipient_ = let_tmp_rhs0_.CiphertextForRecipient
        if not((d_3_CiphertextBlob_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(163,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((d_4_Plaintext_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(164,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((d_5_KeyId_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(165,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((len((d_4_Plaintext_).value)) == (((input).NumberOfBytes).value)):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(166,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_7_decryptInput_: ComAmazonawsKmsTypes.DecryptRequest
        d_7_decryptInput_ = ComAmazonawsKmsTypes.DecryptRequest_DecryptRequest((d_3_CiphertextBlob_).value, (input).EncryptionContext, (input).GrantTokens, Wrappers.Option_Some((d_5_KeyId_).value), Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None())
        default__.BasicDecryptTest(d_7_decryptInput_, (d_4_Plaintext_).value, (input).KeyId)

    @staticmethod
    def BasicGenerateWithoutPlaintextTest(input):
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(default__.TEST__REGION)
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(188,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()
        d_2_retGenerate_: Wrappers.Result
        out1_: Wrappers.Result
        out1_ = (d_1_client_).GenerateDataKeyWithoutPlaintext(input)
        d_2_retGenerate_ = out1_
        if not((d_2_retGenerate_).is_Success):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(192,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        let_tmp_rhs0_ = (d_2_retGenerate_).value
        d_3_CiphertextBlob_ = let_tmp_rhs0_.CiphertextBlob
        d_4_KeyId_ = let_tmp_rhs0_.KeyId
        if not((d_3_CiphertextBlob_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(196,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((d_4_KeyId_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(197,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_5_decryptInput_: ComAmazonawsKmsTypes.DecryptRequest
        d_5_decryptInput_ = ComAmazonawsKmsTypes.DecryptRequest_DecryptRequest((d_3_CiphertextBlob_).value, (input).EncryptionContext, (input).GrantTokens, Wrappers.Option_Some((d_4_KeyId_).value), Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None())
        d_6_ret_: Wrappers.Result
        out2_: Wrappers.Result
        out2_ = (d_1_client_).Decrypt(d_5_decryptInput_)
        d_6_ret_ = out2_
        if not((d_6_ret_).is_Success):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(208,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        let_tmp_rhs1_ = (d_6_ret_).value
        d_7_KeyIdTwo_ = let_tmp_rhs1_.KeyId
        d_8_Plaintext_ = let_tmp_rhs1_.Plaintext
        d_9_EncryptionAlgorithm_ = let_tmp_rhs1_.EncryptionAlgorithm
        d_10_CiphertextBlobTwo_ = let_tmp_rhs1_.CiphertextForRecipient
        if not((d_7_KeyIdTwo_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(211,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not(((d_7_KeyIdTwo_).value) == ((d_4_KeyId_).value)):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(212,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def BasicEncryptTest(input):
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(default__.TEST__REGION)
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(219,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()
        d_2_ret_: Wrappers.Result
        out1_: Wrappers.Result
        out1_ = (d_1_client_).Encrypt(input)
        d_2_ret_ = out1_
        if not((d_2_ret_).is_Success):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(223,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        let_tmp_rhs0_ = (d_2_ret_).value
        d_3_CiphertextBlob_ = let_tmp_rhs0_.CiphertextBlob
        d_4_KeyId_ = let_tmp_rhs0_.KeyId
        d_5_EncryptionAlgorithm_ = let_tmp_rhs0_.EncryptionAlgorithm
        if not((d_3_CiphertextBlob_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(227,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((d_4_KeyId_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(228,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        d_6_decryptInput_: ComAmazonawsKmsTypes.DecryptRequest
        d_6_decryptInput_ = ComAmazonawsKmsTypes.DecryptRequest_DecryptRequest((d_3_CiphertextBlob_).value, (input).EncryptionContext, (input).GrantTokens, Wrappers.Option_Some((d_4_KeyId_).value), (input).EncryptionAlgorithm, Wrappers.Option_None(), Wrappers.Option_None())
        default__.BasicDecryptTest(d_6_decryptInput_, (input).Plaintext, (input).KeyId)

    @staticmethod
    def RegionMatchTest():
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(default__.TEST__REGION)
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(248,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()
        d_2_region_: Wrappers.Option
        d_2_region_ = Com_Amazonaws_Kms.default__.RegionMatch(d_1_client_, default__.TEST__REGION)
        if not(((d_2_region_).is_None) or ((d_2_region_).value)):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(250,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def EmptyStringIsDefaultRegion():
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(_dafny.Seq(""))
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(255,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()

    @staticmethod
    def BasicDeriveSharedSecretTests(input):
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(default__.TEST__REGION)
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(262,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()
        d_2_ret_: Wrappers.Result
        out1_: Wrappers.Result
        out1_ = (d_1_client_).DeriveSharedSecret(ComAmazonawsKmsTypes.DeriveSharedSecretRequest_DeriveSharedSecretRequest((input).KeyId, (input).KeyAgreementAlgorithm, (input).PublicKey, Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None()))
        d_2_ret_ = out1_
        if (d_2_ret_).is_Success:
            let_tmp_rhs0_ = (d_2_ret_).value
            d_3_KeyId_ = let_tmp_rhs0_.KeyId
            d_4_SharedSecret_ = let_tmp_rhs0_.SharedSecret
            d_5_CiphertextForRecipient_ = let_tmp_rhs0_.CiphertextForRecipient
            d_6_KeyAgreementAlgorithm_ = let_tmp_rhs0_.KeyAgreementAlgorithm
            d_7_KeyOrigin_ = let_tmp_rhs0_.KeyOrigin
            if not((d_4_SharedSecret_).is_Some):
                raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(275,6): " + _dafny.string_of(_dafny.Seq("expectation violation")))
            if not((d_3_KeyId_).is_Some):
                raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(276,6): " + _dafny.string_of(_dafny.Seq("expectation violation")))
            if not(((d_3_KeyId_).value) == ((input).KeyId)):
                raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(278,6): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        elif True:
            if not((d_2_ret_).is_Failure):
                raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(281,6): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def GetPublicKeyHelper(input):
        publicKey: _dafny.Seq = None
        d_0_valueOrError0_: Wrappers.Result = None
        out0_: Wrappers.Result
        out0_ = Com_Amazonaws_Kms.default__.KMSClientForRegion(default__.TEST__REGION)
        d_0_valueOrError0_ = out0_
        if not(not((d_0_valueOrError0_).IsFailure())):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(292,18): " + _dafny.string_of(d_0_valueOrError0_))
        d_1_client_: ComAmazonawsKmsTypes.IKMSClient
        d_1_client_ = (d_0_valueOrError0_).Extract()
        d_2_ret_: Wrappers.Result
        out1_: Wrappers.Result
        out1_ = (d_1_client_).GetPublicKey(ComAmazonawsKmsTypes.GetPublicKeyRequest_GetPublicKeyRequest((input).KeyId, (input).GrantTokens))
        d_2_ret_ = out1_
        if not((d_2_ret_).is_Success):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(299,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        let_tmp_rhs0_ = (d_2_ret_).value
        d_3___v1_ = let_tmp_rhs0_.KeyId
        d_4_PublicKey_ = let_tmp_rhs0_.PublicKey
        d_5___v2_ = let_tmp_rhs0_.CustomerMasterKeySpec
        d_6___v3_ = let_tmp_rhs0_.KeySpec
        d_7___v4_ = let_tmp_rhs0_.KeyUsage
        d_8___v5_ = let_tmp_rhs0_.EncryptionAlgorithms
        d_9___v6_ = let_tmp_rhs0_.SigningAlgorithms
        d_10___v7_ = let_tmp_rhs0_.KeyAgreementAlgorithms
        if not((d_4_PublicKey_).is_Some):
            raise _dafny.HaltException("test/TestComAmazonawsKms.dfy(302,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        publicKey = (d_4_PublicKey_).value
        return publicKey
        return publicKey

    @staticmethod
    def DeriveSharedSecretTestSuccess():
        d_0_recipientPublicKey_: _dafny.Seq
        out0_: _dafny.Seq
        out0_ = default__.GetPublicKeyHelper(ComAmazonawsKmsTypes.GetPublicKeyRequest_GetPublicKeyRequest(default__.recipientKmsKey, Wrappers.Option_None()))
        d_0_recipientPublicKey_ = out0_
        default__.BasicDeriveSharedSecretTests(ComAmazonawsKmsTypes.DeriveSharedSecretRequest_DeriveSharedSecretRequest(default__.senderKmsKey, ComAmazonawsKmsTypes.KeyAgreementAlgorithmSpec_ECDH(), d_0_recipientPublicKey_, Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None()))

    @staticmethod
    def DeriveSharedSecretTestFailure():
        d_0_recipientPublicKeyOnWrongCurve_: _dafny.Seq
        out0_: _dafny.Seq
        out0_ = default__.GetPublicKeyHelper(ComAmazonawsKmsTypes.GetPublicKeyRequest_GetPublicKeyRequest(default__.incorrectEccCurveKey, Wrappers.Option_None()))
        d_0_recipientPublicKeyOnWrongCurve_ = out0_
        default__.BasicDeriveSharedSecretTests(ComAmazonawsKmsTypes.DeriveSharedSecretRequest_DeriveSharedSecretRequest(default__.senderKmsKey, ComAmazonawsKmsTypes.KeyAgreementAlgorithmSpec_ECDH(), d_0_recipientPublicKeyOnWrongCurve_, Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_None()))

    @staticmethod
    def CreateNoneForEncryptionContext():
        return Wrappers.Option_None()

    @staticmethod
    def CreateNoneForKeySpec():
        return Wrappers.Option_None()

    @staticmethod
    def CreateNoneForNumberOfBytes():
        return Wrappers.Option_None()

    @staticmethod
    def CreateNoneForGrantTokens():
        return Wrappers.Option_None()

    @staticmethod
    def CreateNoneForDryRun():
        return Wrappers.Option_None()

    @_dafny.classproperty
    def TEST__REGION(instance):
        return _dafny.Seq("us-west-2")
    @_dafny.classproperty
    def keyId(instance):
        return _dafny.Seq("arn:aws:kms:us-west-2:658956600833:key/b3537ef1-d8dc-4780-9f5a-55776cbb2f7f")
    @_dafny.classproperty
    def keyIdGenerateWOPlain(instance):
        return _dafny.Seq("arn:aws:kms:us-west-2:370957321024:key/9d989aa2-2f9c-438c-a745-cc57d3ad0126")
    @_dafny.classproperty
    def failingKeyId(instance):
        return _dafny.Seq("arn:aws:kms:us-west-2:370957321024:key/e20ac7b8-3d95-46ee-b3d5-f5dca6393945")
    @_dafny.classproperty
    def failingInput(instance):
        return ComAmazonawsKmsTypes.GenerateDataKeyWithoutPlaintextRequest_GenerateDataKeyWithoutPlaintextRequest(default__.failingKeyId, Wrappers.Option_None(), Wrappers.Option_None(), Wrappers.Option_Some(32), Wrappers.Option_None(), Wrappers.Option_None())
    @_dafny.classproperty
    def recipientKmsKey(instance):
        return _dafny.Seq("arn:aws:kms:us-west-2:370957321024:key/0265c8e9-5b6a-4055-8f70-63719e09fda5")
    @_dafny.classproperty
    def senderKmsKey(instance):
        return _dafny.Seq("arn:aws:kms:us-west-2:370957321024:key/eabdf483-6be2-4d2d-8ee4-8c2583d416e9")
    @_dafny.classproperty
    def incorrectEccCurveKey(instance):
        return _dafny.Seq("arn:aws:kms:us-west-2:370957321024:key/7f35a704-f4fb-469d-98b1-62a1fa2cc44e")



# Dafny program the_program compiled into Python
import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import module_ as module_
import _dafny as _dafny

try:
    dafnyArgs = [_dafny.Seq(a) for a in sys.argv]
    module_.default__.Test____Main____(dafnyArgs)
except _dafny.HaltException as e:
    _dafny.print("[Program halted] " + e.message + "\n")
    sys.exit(1)



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping

# Module: JSON_Utils_Str

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def OfNat(n, base):
        return JSON_Utils_Str_CharStrConversion.default__.OfNat__any(n, _dafny.Seq((default__.HEX__DIGITS)[:base:]))

    @staticmethod
    def OfInt(n, base):
        return JSON_Utils_Str_CharStrConversion.default__.OfInt__any(n, _dafny.Seq((default__.HEX__DIGITS)[:base:]), '-')

    @staticmethod
    def ToNat(str, base):
        return JSON_Utils_Str_CharStrConversion.default__.ToNat__any(str, base, default__.HEX__TABLE)

    @staticmethod
    def ToInt(str, base):
        return JSON_Utils_Str_CharStrConversion.default__.ToInt__any(str, '-', base, default__.HEX__TABLE)

    @staticmethod
    def EscapeQuotes(str):
        return JSON_Utils_Str_CharStrEscaping.default__.Escape(str, _dafny.Set({'\"', '\''}), '\\')

    @staticmethod
    def UnescapeQuotes(str):
        return JSON_Utils_Str_CharStrEscaping.default__.Unescape(str, '\\')

    @staticmethod
    def Test():
        if not((default__.OfInt(0, 10)) == (_dafny.Seq("0"))):
            raise _dafny.HaltException("/codebuild/output/src3189093257/src/github.com/aws/aws-cryptographic-material-providers-library/libraries/src/JSON/Utils/Str.dfy(229,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((default__.OfInt(3, 10)) == (_dafny.Seq("3"))):
            raise _dafny.HaltException("/codebuild/output/src3189093257/src/github.com/aws/aws-cryptographic-material-providers-library/libraries/src/JSON/Utils/Str.dfy(230,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((default__.OfInt(302, 10)) == (_dafny.Seq("302"))):
            raise _dafny.HaltException("/codebuild/output/src3189093257/src/github.com/aws/aws-cryptographic-material-providers-library/libraries/src/JSON/Utils/Str.dfy(231,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((default__.OfInt(-3, 10)) == (_dafny.Seq("-3"))):
            raise _dafny.HaltException("/codebuild/output/src3189093257/src/github.com/aws/aws-cryptographic-material-providers-library/libraries/src/JSON/Utils/Str.dfy(232,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))
        if not((default__.OfInt(-302, 10)) == (_dafny.Seq("-302"))):
            raise _dafny.HaltException("/codebuild/output/src3189093257/src/github.com/aws/aws-cryptographic-material-providers-library/libraries/src/JSON/Utils/Str.dfy(233,4): " + _dafny.string_of(_dafny.Seq("expectation violation")))

    @staticmethod
    def OfBool(b):
        if b:
            return _dafny.Seq("true")
        elif True:
            return _dafny.Seq("false")

    @staticmethod
    def OfChar(c):
        return _dafny.Seq([c])

    @staticmethod
    def Join(sep, strs):
        d_0___accumulator_ = _dafny.Seq([])
        while True:
            with _dafny.label():
                if (len(strs)) == (0):
                    return (d_0___accumulator_) + (_dafny.Seq(""))
                elif (len(strs)) == (1):
                    return (d_0___accumulator_) + ((strs)[0])
                elif True:
                    d_0___accumulator_ = (d_0___accumulator_) + (((strs)[0]) + (sep))
                    in0_ = sep
                    in1_ = _dafny.Seq((strs)[1::])
                    sep = in0_
                    strs = in1_
                    raise _dafny.TailCall()
                break

    @staticmethod
    def Concat(strs):
        d_0___accumulator_ = _dafny.Seq([])
        while True:
            with _dafny.label():
                if (len(strs)) == (0):
                    return (d_0___accumulator_) + (_dafny.Seq(""))
                elif True:
                    d_0___accumulator_ = (d_0___accumulator_) + ((strs)[0])
                    in0_ = _dafny.Seq((strs)[1::])
                    strs = in0_
                    raise _dafny.TailCall()
                break

    @_dafny.classproperty
    def HEX__DIGITS(instance):
        return _dafny.Seq("0123456789ABCDEF")
    @_dafny.classproperty
    def HEX__TABLE(instance):
        return _dafny.Map({'0': 0, '1': 1, '2': 2, '3': 3, '4': 4, '5': 5, '6': 6, '7': 7, '8': 8, '9': 9, 'a': 10, 'b': 11, 'c': 12, 'd': 13, 'e': 14, 'f': 15, 'A': 10, 'B': 11, 'C': 12, 'D': 13, 'E': 14, 'F': 15})



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time

# Module: Streams


class SeqReader:
    def  __init__(self):
        self.pos: int = int(0)
        self._data: _dafny.Seq = _dafny.Seq({})
        pass

    def __dafnystr__(self) -> str:
        return "Streams.SeqReader"
    def ctor__(self, s):
        (self)._data = s
        (self).pos = 0

    def ReadElements(self, n):
        elems: _dafny.Seq = _dafny.Seq({})
        elems = _dafny.Seq((_dafny.Seq(((self).data)[self.pos::]))[:n:])
        (self).pos = (self.pos) + (n)
        elems = elems
        return elems
        return elems

    def ReadExact(self, n):
        res: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        if (n) > ((len((self).data)) - (self.pos)):
            res = Wrappers.Result_Failure(_dafny.Seq("IO Error: Not enough elements left on stream."))
            return res
        elif True:
            d_0_elements_: _dafny.Seq
            out0_: _dafny.Seq
            out0_ = (self).ReadElements(n)
            d_0_elements_ = out0_
            res = Wrappers.Result_Success(d_0_elements_)
            return res
        return res

    @property
    def data(self):
        return self._data

class ByteReader:
    def  __init__(self):
        self._reader: SeqReader = None
        pass

    def __dafnystr__(self) -> str:
        return "Streams.ByteReader"
    def ctor__(self, s):
        d_0_mr_: SeqReader
        nw0_ = SeqReader()
        nw0_.ctor__(s)
        d_0_mr_ = nw0_
        (self)._reader = d_0_mr_

    def ReadByte(self):
        res: Wrappers.Result = Wrappers.Result.default(BoundedInts.uint8.default)()
        d_0_valueOrError0_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        out0_: Wrappers.Result
        out0_ = ((self).reader).ReadExact(1)
        d_0_valueOrError0_ = out0_
        if (d_0_valueOrError0_).IsFailure():
            res = (d_0_valueOrError0_).PropagateFailure()
            return res
        d_1_bytes_: _dafny.Seq
        d_1_bytes_ = (d_0_valueOrError0_).Extract()
        res = Wrappers.Result_Success((d_1_bytes_)[0])
        return res
        return res

    def ReadBytes(self, n):
        res: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        d_0_valueOrError0_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        out0_: Wrappers.Result
        out0_ = ((self).reader).ReadExact(n)
        d_0_valueOrError0_ = out0_
        if (d_0_valueOrError0_).IsFailure():
            res = (d_0_valueOrError0_).PropagateFailure()
            return res
        d_1_bytes_: _dafny.Seq
        d_1_bytes_ = (d_0_valueOrError0_).Extract()
        res = Wrappers.Result_Success(d_1_bytes_)
        return res
        return res

    def ReadUInt16(self):
        res: Wrappers.Result = Wrappers.Result.default(BoundedInts.uint16.default)()
        d_0_valueOrError0_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        out0_: Wrappers.Result
        out0_ = ((self).reader).ReadExact(2)
        d_0_valueOrError0_ = out0_
        if (d_0_valueOrError0_).IsFailure():
            res = (d_0_valueOrError0_).PropagateFailure()
            return res
        d_1_bytes_: _dafny.Seq
        d_1_bytes_ = (d_0_valueOrError0_).Extract()
        d_2_n_: int
        d_2_n_ = StandardLibrary_UInt.default__.SeqToUInt16(d_1_bytes_)
        res = Wrappers.Result_Success(d_2_n_)
        return res
        return res

    def ReadUInt32(self):
        res: Wrappers.Result = Wrappers.Result.default(BoundedInts.uint32.default)()
        d_0_valueOrError0_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        out0_: Wrappers.Result
        out0_ = ((self).reader).ReadExact(4)
        d_0_valueOrError0_ = out0_
        if (d_0_valueOrError0_).IsFailure():
            res = (d_0_valueOrError0_).PropagateFailure()
            return res
        d_1_bytes_: _dafny.Seq
        d_1_bytes_ = (d_0_valueOrError0_).Extract()
        d_2_n_: int
        d_2_n_ = StandardLibrary_UInt.default__.SeqToUInt32(d_1_bytes_)
        res = Wrappers.Result_Success(d_2_n_)
        return res
        return res

    def ReadUInt64(self):
        res: Wrappers.Result = Wrappers.Result.default(BoundedInts.uint64.default)()
        d_0_valueOrError0_: Wrappers.Result = Wrappers.Result.default(_dafny.Seq)()
        out0_: Wrappers.Result
        out0_ = ((self).reader).ReadExact(8)
        d_0_valueOrError0_ = out0_
        if (d_0_valueOrError0_).IsFailure():
            res = (d_0_valueOrError0_).PropagateFailure()
            return res
        d_1_bytes_: _dafny.Seq
        d_1_bytes_ = (d_0_valueOrError0_).Extract()
        d_2_n_: int
        d_2_n_ = StandardLibrary_UInt.default__.SeqToUInt64(d_1_bytes_)
        res = Wrappers.Result_Success(d_2_n_)
        return res
        return res

    def IsDoneReading(self):
        b: bool = False
        b = (len(((self).reader).data)) == ((self).reader.pos)
        return b
        return b

    def GetSizeRead(self):
        n: int = int(0)
        n = (self).reader.pos
        return n
        return n

    @property
    def reader(self):
        return self._reader

class SeqWriter:
    def  __init__(self):
        self.data: _dafny.Seq = _dafny.Seq({})
        pass

    def __dafnystr__(self) -> str:
        return "Streams.SeqWriter"
    def ctor__(self):
        (self).data = _dafny.Seq([])

    def WriteElements(self, elems):
        n: int = int(0)
        (self).data = (self.data) + (elems)
        n = len(elems)
        return n
        return n


class ByteWriter:
    def  __init__(self):
        self._writer: SeqWriter = None
        pass

    def __dafnystr__(self) -> str:
        return "Streams.ByteWriter"
    def ctor__(self):
        d_0_mw_: SeqWriter
        nw0_ = SeqWriter()
        nw0_.ctor__()
        d_0_mw_ = nw0_
        (self)._writer = d_0_mw_

    def WriteByte(self, n):
        r: int = int(0)
        out0_: int
        out0_ = ((self).writer).WriteElements(_dafny.Seq([n]))
        r = out0_
        return r

    def WriteBytes(self, s):
        r: int = int(0)
        out0_: int
        out0_ = ((self).writer).WriteElements(s)
        r = out0_
        return r

    def WriteUInt16(self, n):
        r: int = int(0)
        out0_: int
        out0_ = ((self).writer).WriteElements(StandardLibrary_UInt.default__.UInt16ToSeq(n))
        r = out0_
        return r

    def WriteUInt32(self, n):
        r: int = int(0)
        out0_: int
        out0_ = ((self).writer).WriteElements(StandardLibrary_UInt.default__.UInt32ToSeq(n))
        r = out0_
        return r

    def GetDataWritten(self):
        return (self).writer.data

    def GetSizeWritten(self):
        return len((self).writer.data)

    @property
    def writer(self):
        return self._writer



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID
import smithy_dafny_standard_library.internaldafny.generated.UTF8 as UTF8
import smithy_dafny_standard_library.internaldafny.generated.Time as Time
import smithy_dafny_standard_library.internaldafny.generated.Streams as Streams
import smithy_dafny_standard_library.internaldafny.generated.Sorting as Sorting
import smithy_dafny_standard_library.internaldafny.generated.SortedSets as SortedSets
import smithy_dafny_standard_library.internaldafny.generated.HexStrings as HexStrings
import smithy_dafny_standard_library.internaldafny.generated.GetOpt as GetOpt
import smithy_dafny_standard_library.internaldafny.generated.FloatCompare as FloatCompare
import smithy_dafny_standard_library.internaldafny.generated.ConcurrentCall as ConcurrentCall
import smithy_dafny_standard_library.internaldafny.generated.Base64 as Base64
import smithy_dafny_standard_library.internaldafny.generated.Base64Lemmas as Base64Lemmas
import smithy_dafny_standard_library.internaldafny.generated.Actions as Actions
import smithy_dafny_standard_library.internaldafny.generated.DafnyLibraries as DafnyLibraries
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Core as JSON_Utils_Views_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views_Writers as JSON_Utils_Views_Writers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Views as JSON_Utils_Views
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Core as JSON_Utils_Lexers_Core
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers_Strings as JSON_Utils_Lexers_Strings
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Lexers as JSON_Utils_Lexers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Cursors as JSON_Utils_Cursors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Parsers as JSON_Utils_Parsers
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrConversion as JSON_Utils_Str_CharStrConversion
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str_CharStrEscaping as JSON_Utils_Str_CharStrEscaping
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Str as JSON_Utils_Str
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Seq as JSON_Utils_Seq
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils_Vectors as JSON_Utils_Vectors
import smithy_dafny_standard_library.internaldafny.generated.JSON_Utils as JSON_Utils

# Module: JSON_Errors


class DeserializationError:
    @classmethod
    def default(cls, ):
        return lambda: DeserializationError_UnterminatedSequence()
    def __ne__(self, __o: object) -> bool:
        return not self.__eq__(__o)
    @property
    def is_UnterminatedSequence(self) -> bool:
        return isinstance(self, DeserializationError_UnterminatedSequence)
    @property
    def is_UnsupportedEscape(self) -> bool:
        return isinstance(self, DeserializationError_UnsupportedEscape)
    @property
    def is_EscapeAtEOS(self) -> bool:
        return isinstance(self, DeserializationError_EscapeAtEOS)
    @property
    def is_EmptyNumber(self) -> bool:
        return isinstance(self, DeserializationError_EmptyNumber)
    @property
    def is_ExpectingEOF(self) -> bool:
        return isinstance(self, DeserializationError_ExpectingEOF)
    @property
    def is_IntOverflow(self) -> bool:
        return isinstance(self, DeserializationError_IntOverflow)
    @property
    def is_ReachedEOF(self) -> bool:
        return isinstance(self, DeserializationError_ReachedEOF)
    @property
    def is_ExpectingByte(self) -> bool:
        return isinstance(self, DeserializationError_ExpectingByte)
    @property
    def is_ExpectingAnyByte(self) -> bool:
        return isinstance(self, DeserializationError_ExpectingAnyByte)
    @property
    def is_InvalidUnicode(self) -> bool:
        return isinstance(self, DeserializationError_InvalidUnicode)
    def ToString(self):
        source0_ = self
        if True:
            if source0_.is_UnterminatedSequence:
                return _dafny.Seq("Unterminated sequence")
        if True:
            if source0_.is_UnsupportedEscape:
                d_0_str_ = source0_.str
                return (_dafny.Seq("Unsupported escape sequence: ")) + (d_0_str_)
        if True:
            if source0_.is_EscapeAtEOS:
                return _dafny.Seq("Escape character at end of string")
        if True:
            if source0_.is_EmptyNumber:
                return _dafny.Seq("Number must contain at least one digit")
        if True:
            if source0_.is_ExpectingEOF:
                return _dafny.Seq("Expecting EOF")
        if True:
            if source0_.is_IntOverflow:
                return _dafny.Seq("Input length does not fit in a 32-bit counter")
        if True:
            if source0_.is_ReachedEOF:
                return _dafny.Seq("Reached EOF")
        if True:
            if source0_.is_ExpectingByte:
                d_1_b0_ = source0_.expected
                d_2_b_ = source0_.b
                d_3_c_ = (((_dafny.Seq("'")) + (_dafny.Seq([chr(d_2_b_)]))) + (_dafny.Seq("'")) if (d_2_b_) > (0) else _dafny.Seq("EOF"))
                return (((_dafny.Seq("Expecting '")) + (_dafny.Seq([chr(d_1_b0_)]))) + (_dafny.Seq("', read "))) + (d_3_c_)
        if True:
            if source0_.is_ExpectingAnyByte:
                d_4_bs0_ = source0_.expected__sq
                d_5_b_ = source0_.b
                d_6_c_ = (((_dafny.Seq("'")) + (_dafny.Seq([chr(d_5_b_)]))) + (_dafny.Seq("'")) if (d_5_b_) > (0) else _dafny.Seq("EOF"))
                d_7_c0s_ = _dafny.Seq([chr((d_4_bs0_)[d_8_idx_]) for d_8_idx_ in range(len(d_4_bs0_))])
                return (((_dafny.Seq("Expecting one of '")) + (d_7_c0s_)) + (_dafny.Seq("', read "))) + (d_6_c_)
        if True:
            return _dafny.Seq("Invalid Unicode sequence")


class DeserializationError_UnterminatedSequence(DeserializationError, NamedTuple('UnterminatedSequence', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.UnterminatedSequence'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_UnterminatedSequence)
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_UnsupportedEscape(DeserializationError, NamedTuple('UnsupportedEscape', [('str', Any)])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.UnsupportedEscape({_dafny.string_of(self.str)})'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_UnsupportedEscape) and self.str == __o.str
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_EscapeAtEOS(DeserializationError, NamedTuple('EscapeAtEOS', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.EscapeAtEOS'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_EscapeAtEOS)
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_EmptyNumber(DeserializationError, NamedTuple('EmptyNumber', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.EmptyNumber'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_EmptyNumber)
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_ExpectingEOF(DeserializationError, NamedTuple('ExpectingEOF', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.ExpectingEOF'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_ExpectingEOF)
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_IntOverflow(DeserializationError, NamedTuple('IntOverflow', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.IntOverflow'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_IntOverflow)
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_ReachedEOF(DeserializationError, NamedTuple('ReachedEOF', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.ReachedEOF'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_ReachedEOF)
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_ExpectingByte(DeserializationError, NamedTuple('ExpectingByte', [('expected', Any), ('b', Any)])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.ExpectingByte({_dafny.string_of(self.expected)}, {_dafny.string_of(self.b)})'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_ExpectingByte) and self.expected == __o.expected and self.b == __o.b
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_ExpectingAnyByte(DeserializationError, NamedTuple('ExpectingAnyByte', [('expected__sq', Any), ('b', Any)])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.ExpectingAnyByte({_dafny.string_of(self.expected__sq)}, {_dafny.string_of(self.b)})'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_ExpectingAnyByte) and self.expected__sq == __o.expected__sq and self.b == __o.b
    def __hash__(self) -> int:
        return super().__hash__()

class DeserializationError_InvalidUnicode(DeserializationError, NamedTuple('InvalidUnicode', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.DeserializationError.InvalidUnicode'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, DeserializationError_InvalidUnicode)
    def __hash__(self) -> int:
        return super().__hash__()


class SerializationError:
    @classmethod
    def default(cls, ):
        return lambda: SerializationError_OutOfMemory()
    def __ne__(self, __o: object) -> bool:
        return not self.__eq__(__o)
    @property
    def is_OutOfMemory(self) -> bool:
        return isinstance(self, SerializationError_OutOfMemory)
    @property
    def is_IntTooLarge(self) -> bool:
        return isinstance(self, SerializationError_IntTooLarge)
    @property
    def is_StringTooLong(self) -> bool:
        return isinstance(self, SerializationError_StringTooLong)
    @property
    def is_InvalidUnicode(self) -> bool:
        return isinstance(self, SerializationError_InvalidUnicode)
    def ToString(self):
        source0_ = self
        if True:
            if source0_.is_OutOfMemory:
                return _dafny.Seq("Out of memory")
        if True:
            if source0_.is_IntTooLarge:
                d_0_i_ = source0_.i
                return (_dafny.Seq("Integer too large: ")) + (JSON_Utils_Str.default__.OfInt(d_0_i_, 10))
        if True:
            if source0_.is_StringTooLong:
                d_1_s_ = source0_.s
                return (_dafny.Seq("String too long: ")) + (d_1_s_)
        if True:
            return _dafny.Seq("Invalid Unicode sequence")


class SerializationError_OutOfMemory(SerializationError, NamedTuple('OutOfMemory', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.SerializationError.OutOfMemory'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, SerializationError_OutOfMemory)
    def __hash__(self) -> int:
        return super().__hash__()

class SerializationError_IntTooLarge(SerializationError, NamedTuple('IntTooLarge', [('i', Any)])):
    def __dafnystr__(self) -> str:
        return f'Errors.SerializationError.IntTooLarge({_dafny.string_of(self.i)})'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, SerializationError_IntTooLarge) and self.i == __o.i
    def __hash__(self) -> int:
        return super().__hash__()

class SerializationError_StringTooLong(SerializationError, NamedTuple('StringTooLong', [('s', Any)])):
    def __dafnystr__(self) -> str:
        return f'Errors.SerializationError.StringTooLong({_dafny.string_of(self.s)})'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, SerializationError_StringTooLong) and self.s == __o.s
    def __hash__(self) -> int:
        return super().__hash__()

class SerializationError_InvalidUnicode(SerializationError, NamedTuple('InvalidUnicode', [])):
    def __dafnystr__(self) -> str:
        return f'Errors.SerializationError.InvalidUnicode'
    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, SerializationError_InvalidUnicode)
    def __hash__(self) -> int:
        return super().__hash__()




# Dafny program the_program compiled into Python
import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import aws_cryptography_internal_kms.internaldafny.generated.module_ as module_
import _dafny as _dafny




import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop

# Module: StandardLibrary_UInt

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def UInt8Less(a, b):
        return (a) < (b)

    @staticmethod
    def HasUint16Len(s):
        return (len(s)) < (default__.UINT16__LIMIT)

    @staticmethod
    def HasUint32Len(s):
        return (len(s)) < (default__.UINT32__LIMIT)

    @staticmethod
    def HasUint64Len(s):
        return (len(s)) < (default__.UINT64__LIMIT)

    @staticmethod
    def UInt16ToSeq(x):
        d_0_b0_ = _dafny.euclidian_division(x, 256)
        d_1_b1_ = _dafny.euclidian_modulus(x, 256)
        return _dafny.Seq([d_0_b0_, d_1_b1_])

    @staticmethod
    def SeqToUInt16(s):
        d_0_x0_ = ((s)[0]) * (256)
        return (d_0_x0_) + ((s)[1])

    @staticmethod
    def UInt32ToSeq(x):
        d_0_b0_ = _dafny.euclidian_division(x, 16777216)
        d_1_x0_ = (x) - ((d_0_b0_) * (16777216))
        d_2_b1_ = _dafny.euclidian_division(d_1_x0_, 65536)
        d_3_x1_ = (d_1_x0_) - ((d_2_b1_) * (65536))
        d_4_b2_ = _dafny.euclidian_division(d_3_x1_, 256)
        d_5_b3_ = _dafny.euclidian_modulus(d_3_x1_, 256)
        return _dafny.Seq([d_0_b0_, d_2_b1_, d_4_b2_, d_5_b3_])

    @staticmethod
    def SeqToUInt32(s):
        d_0_x0_ = ((s)[0]) * (16777216)
        d_1_x1_ = (d_0_x0_) + (((s)[1]) * (65536))
        d_2_x2_ = (d_1_x1_) + (((s)[2]) * (256))
        return (d_2_x2_) + ((s)[3])

    @staticmethod
    def UInt64ToSeq(x):
        d_0_b0_ = _dafny.euclidian_division(x, 72057594037927936)
        d_1_x0_ = (x) - ((d_0_b0_) * (72057594037927936))
        d_2_b1_ = _dafny.euclidian_division(d_1_x0_, 281474976710656)
        d_3_x1_ = (d_1_x0_) - ((d_2_b1_) * (281474976710656))
        d_4_b2_ = _dafny.euclidian_division(d_3_x1_, 1099511627776)
        d_5_x2_ = (d_3_x1_) - ((d_4_b2_) * (1099511627776))
        d_6_b3_ = _dafny.euclidian_division(d_5_x2_, 4294967296)
        d_7_x3_ = (d_5_x2_) - ((d_6_b3_) * (4294967296))
        d_8_b4_ = _dafny.euclidian_division(d_7_x3_, 16777216)
        d_9_x4_ = (d_7_x3_) - ((d_8_b4_) * (16777216))
        d_10_b5_ = _dafny.euclidian_division(d_9_x4_, 65536)
        d_11_x5_ = (d_9_x4_) - ((d_10_b5_) * (65536))
        d_12_b6_ = _dafny.euclidian_division(d_11_x5_, 256)
        d_13_b7_ = _dafny.euclidian_modulus(d_11_x5_, 256)
        return _dafny.Seq([d_0_b0_, d_2_b1_, d_4_b2_, d_6_b3_, d_8_b4_, d_10_b5_, d_12_b6_, d_13_b7_])

    @staticmethod
    def SeqToUInt64(s):
        d_0_x0_ = ((s)[0]) * (72057594037927936)
        d_1_x1_ = (d_0_x0_) + (((s)[1]) * (281474976710656))
        d_2_x2_ = (d_1_x1_) + (((s)[2]) * (1099511627776))
        d_3_x3_ = (d_2_x2_) + (((s)[3]) * (4294967296))
        d_4_x4_ = (d_3_x3_) + (((s)[4]) * (16777216))
        d_5_x5_ = (d_4_x4_) + (((s)[5]) * (65536))
        d_6_x6_ = (d_5_x5_) + (((s)[6]) * (256))
        d_7_x_ = (d_6_x6_) + ((s)[7])
        return d_7_x_

    @_dafny.classproperty
    def UINT16__LIMIT(instance):
        return (BoundedInts.default__.UINT16__MAX) + (1)
    @_dafny.classproperty
    def UINT32__LIMIT(instance):
        return (BoundedInts.default__.UINT32__MAX) + (1)
    @_dafny.classproperty
    def UINT64__LIMIT(instance):
        return (BoundedInts.default__.UINT64__MAX) + (1)
    @_dafny.classproperty
    def INT32__MAX__LIMIT(instance):
        return BoundedInts.default__.INT32__MAX
    @_dafny.classproperty
    def INT64__MAX__LIMIT(instance):
        return BoundedInts.default__.INT64__MAX

class seq16:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return _dafny.Seq({})
    def _Is(source__):
        d_0_s_: _dafny.Seq = source__
        return default__.HasUint16Len(d_0_s_)

class seq32:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return _dafny.Seq({})
    def _Is(source__):
        d_1_s_: _dafny.Seq = source__
        return default__.HasUint32Len(d_1_s_)

class seq64:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return _dafny.Seq({})
    def _Is(source__):
        d_2_s_: _dafny.Seq = source__
        return default__.HasUint64Len(d_2_s_)



import sys
from typing import Callable, Any, TypeVar, NamedTuple
from math import floor
from itertools import count

import smithy_dafny_standard_library.internaldafny.generated.module_ as module_
import _dafny as _dafny
import System_ as System_
import smithy_dafny_standard_library.internaldafny.generated.Wrappers as Wrappers
import smithy_dafny_standard_library.internaldafny.generated.Relations as Relations
import smithy_dafny_standard_library.internaldafny.generated.Seq_MergeSort as Seq_MergeSort
import smithy_dafny_standard_library.internaldafny.generated.Math as Math
import smithy_dafny_standard_library.internaldafny.generated.Seq as Seq
import smithy_dafny_standard_library.internaldafny.generated.BoundedInts as BoundedInts
import smithy_dafny_standard_library.internaldafny.generated.Unicode as Unicode
import smithy_dafny_standard_library.internaldafny.generated.Functions as Functions
import smithy_dafny_standard_library.internaldafny.generated.Utf8EncodingForm as Utf8EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.Utf16EncodingForm as Utf16EncodingForm
import smithy_dafny_standard_library.internaldafny.generated.UnicodeStrings as UnicodeStrings
import smithy_dafny_standard_library.internaldafny.generated.FileIO as FileIO
import smithy_dafny_standard_library.internaldafny.generated.GeneralInternals as GeneralInternals
import smithy_dafny_standard_library.internaldafny.generated.MulInternalsNonlinear as MulInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.MulInternals as MulInternals
import smithy_dafny_standard_library.internaldafny.generated.Mul as Mul
import smithy_dafny_standard_library.internaldafny.generated.ModInternalsNonlinear as ModInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.DivInternalsNonlinear as DivInternalsNonlinear
import smithy_dafny_standard_library.internaldafny.generated.ModInternals as ModInternals
import smithy_dafny_standard_library.internaldafny.generated.DivInternals as DivInternals
import smithy_dafny_standard_library.internaldafny.generated.DivMod as DivMod
import smithy_dafny_standard_library.internaldafny.generated.Power as Power
import smithy_dafny_standard_library.internaldafny.generated.Logarithm as Logarithm
import smithy_dafny_standard_library.internaldafny.generated.StandardLibraryInterop as StandardLibraryInterop
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_UInt as StandardLibrary_UInt
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary_String as StandardLibrary_String
import smithy_dafny_standard_library.internaldafny.generated.StandardLibrary as StandardLibrary
import smithy_dafny_standard_library.internaldafny.generated.UUID as UUID

# Module: UTF8

class default__:
    def  __init__(self):
        pass

    @staticmethod
    def CreateEncodeSuccess(bytes):
        return Wrappers.Result_Success(bytes)

    @staticmethod
    def CreateEncodeFailure(error):
        return Wrappers.Result_Failure(error)

    @staticmethod
    def CreateDecodeSuccess(s):
        return Wrappers.Result_Success(s)

    @staticmethod
    def CreateDecodeFailure(error):
        return Wrappers.Result_Failure(error)

    @staticmethod
    def IsASCIIString(s):
        def lambda0_(forall_var_0_):
            d_0_i_: int = forall_var_0_
            return not (((0) <= (d_0_i_)) and ((d_0_i_) < (len(s)))) or ((ord((s)[d_0_i_])) < (128))

        return _dafny.quantifier(_dafny.IntegerRange(0, len(s)), True, lambda0_)

    @staticmethod
    def EncodeAscii(s):
        d_0___accumulator_ = _dafny.Seq([])
        while True:
            with _dafny.label():
                if (len(s)) == (0):
                    return (d_0___accumulator_) + (_dafny.Seq([]))
                elif True:
                    d_1_x_ = _dafny.Seq([ord((s)[0])])
                    d_0___accumulator_ = (d_0___accumulator_) + (d_1_x_)
                    in0_ = _dafny.Seq((s)[1::])
                    s = in0_
                    raise _dafny.TailCall()
                break

    @staticmethod
    def Uses1Byte(s):
        return ((0) <= ((s)[0])) and (((s)[0]) <= (127))

    @staticmethod
    def Uses2Bytes(s):
        return (((194) <= ((s)[0])) and (((s)[0]) <= (223))) and (((128) <= ((s)[1])) and (((s)[1]) <= (191)))

    @staticmethod
    def Uses3Bytes(s):
        return (((((((s)[0]) == (224)) and (((160) <= ((s)[1])) and (((s)[1]) <= (191)))) and (((128) <= ((s)[2])) and (((s)[2]) <= (191)))) or (((((225) <= ((s)[0])) and (((s)[0]) <= (236))) and (((128) <= ((s)[1])) and (((s)[1]) <= (191)))) and (((128) <= ((s)[2])) and (((s)[2]) <= (191))))) or (((((s)[0]) == (237)) and (((128) <= ((s)[1])) and (((s)[1]) <= (159)))) and (((128) <= ((s)[2])) and (((s)[2]) <= (191))))) or (((((238) <= ((s)[0])) and (((s)[0]) <= (239))) and (((128) <= ((s)[1])) and (((s)[1]) <= (191)))) and (((128) <= ((s)[2])) and (((s)[2]) <= (191))))

    @staticmethod
    def Uses4Bytes(s):
        return (((((((s)[0]) == (240)) and (((144) <= ((s)[1])) and (((s)[1]) <= (191)))) and (((128) <= ((s)[2])) and (((s)[2]) <= (191)))) and (((128) <= ((s)[3])) and (((s)[3]) <= (191)))) or ((((((241) <= ((s)[0])) and (((s)[0]) <= (243))) and (((128) <= ((s)[1])) and (((s)[1]) <= (191)))) and (((128) <= ((s)[2])) and (((s)[2]) <= (191)))) and (((128) <= ((s)[3])) and (((s)[3]) <= (191))))) or ((((((s)[0]) == (244)) and (((128) <= ((s)[1])) and (((s)[1]) <= (143)))) and (((128) <= ((s)[2])) and (((s)[2]) <= (191)))) and (((128) <= ((s)[3])) and (((s)[3]) <= (191))))

    @staticmethod
    def ValidUTF8Range(a, lo, hi):
        while True:
            with _dafny.label():
                if (lo) == (hi):
                    return True
                elif True:
                    d_0_r_ = _dafny.Seq((a)[lo:hi:])
                    if default__.Uses1Byte(d_0_r_):
                        in0_ = a
                        in1_ = (lo) + (1)
                        in2_ = hi
                        a = in0_
                        lo = in1_
                        hi = in2_
                        raise _dafny.TailCall()
                    elif ((2) <= (len(d_0_r_))) and (default__.Uses2Bytes(d_0_r_)):
                        in3_ = a
                        in4_ = (lo) + (2)
                        in5_ = hi
                        a = in3_
                        lo = in4_
                        hi = in5_
                        raise _dafny.TailCall()
                    elif ((3) <= (len(d_0_r_))) and (default__.Uses3Bytes(d_0_r_)):
                        in6_ = a
                        in7_ = (lo) + (3)
                        in8_ = hi
                        a = in6_
                        lo = in7_
                        hi = in8_
                        raise _dafny.TailCall()
                    elif ((4) <= (len(d_0_r_))) and (default__.Uses4Bytes(d_0_r_)):
                        in9_ = a
                        in10_ = (lo) + (4)
                        in11_ = hi
                        a = in9_
                        lo = in10_
                        hi = in11_
                        raise _dafny.TailCall()
                    elif True:
                        return False
                break

    @staticmethod
    def ValidUTF8Seq(s):
        return default__.ValidUTF8Range(s, 0, len(s))


class ValidUTF8Bytes:
    def  __init__(self):
        pass

    @staticmethod
    def default():
        return _dafny.Seq([])
    def _Is(source__):
        d_0_i_: _dafny.Seq = source__
        return default__.ValidUTF8Seq(d_0_i_)
